import numpy as np
import scipy.special as sp
from src import extraspecial

def acoustics_Mie_a(n, ka, rho1, beta1):
    """
        n - multipole order
        ka - size parameter in host media
        rho1 - relative density
        beta1 - relative compressibility
    """
    gamma = np.sqrt(beta1/rho1)
    k1a = ka * np.sqrt(beta1*rho1)
    jn1 = sp.spherical_jn(n, k1a)
    jn = sp.spherical_jn(n, ka)
    jn1p = sp.spherical_jn(n, k1a, 1)
    jnp = sp.spherical_jn(n, ka, 1)
    extraspecial.spherical_h1(n, ka)
    hn = extraspecial.spherical_h1(n, ka)
    hnp = extraspecial.spherical_h1p(n, ka)
    up = (gamma * jn1p * jn - jn1 * jnp)
    down = (jn1 * hnp - gamma * jn1p * hn)
    
    ans = np.where(
        down == 0,
        0,
        up/down
    )
    return ans
    
    
def acoustics_scattering_cross_section(k, a, rho_rel, beta_rel, nmin=0, nmax=50):
    ka = a * k
    
    sigma_sc = np.zeros(ka.size, dtype=np.float64)
    sigma_sc_n = np.zeros([nmax+1 - nmin, ka.size])
    
    for n in range(nmin, nmax+1):
        an = acoustics_Mie_a(n, ka, rho_rel, beta_rel)
        sigma_sc_n[n, :] = 4*np.pi / k**2 * (2*n+1) * np.abs(an**2)
        
    sigma_sc = np.sum(sigma_sc_n, axis=0)
    
    return sigma_sc, sigma_sc_n


import scipy.special as sp


def spherical_h1(n, z):
    return sp.spherical_jn(n, z) + 1j * sp.spherical_yn(n, z)


def spherical_h1p(n, z):
    return sp.spherical_jn(n, z, 1) + 1j * sp.spherical_yn(n, z, 1)


def spherical_jnpp(n, z):
    # slower option is to
    #     spherical_jn(n-1, z, 1) + (n+1)/z**2 * spherical_jn(n, z) - (n+1)/z * spherical_jn(n, z, 1)
    # but this is faster
    return n/(2*n+1) * sp.spherical_jn(n-1, z, 1) - (n+1)/(2*n+1) * sp.spherical_jn(n+1, z, 1)


def spherical_ynpp(n, z):
    return n/(2*n+1) * sp.spherical_yn(n-1, z, 1) - (n+1)/(2*n+1) * sp.spherical_yn(n+1, z, 1)


def spherical_h1pp(n, z):
    return spherical_jnpp(n, z) + 1j * spherical_ynpp(n, z)


def spherical_h2(n, z):
    return sp.spherical_jn(n, z) - 1j * sp.spherical_yn(n, z)


def spherical_h2p(n, z):
    return sp.spherical_jn(n, z, 1) - 1j * sp.spherical_yn(n, z, 1)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mielib']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'mielib',
    'version': '0.1.1',
    'description': '',
    'long_description': '# MieLib\n\nLibrary which contains many Mie-ralated functions in optics and acoustics. In particular:\n- Scattering Mie coefficients for isotropic spheres:\n    -  Acoustic Mie coefficients based on Phys. Rev. Lett. 123, 183901 (2019) (see SM)\n    - Optics Mie coefficients based on Bohren Huffmann book.\n- Scalar spherical harmonics\n- Vector spherical harmonics (complex and real)\n\n\n## Credits\nIvan Toftul \n\n`toftul.ivan@gmail.com`\n',
    'author': 'Ivan Toftul',
    'author_email': 'toftul.ivan@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
